Package: rerddap
Title: General Purpose Client for 'ERDDAP™' Servers
Description: General purpose R client for 'ERDDAP™' servers. Includes
    functions to search for 'datasets', get summary information on
    'datasets', and fetch 'datasets', in either 'csv' or 'netCDF' format.
    'ERDDAP™' information: 
    <https://upwell.pfeg.noaa.gov/erddap/information.html>.
Version: 1.3.0
Date: 2026-06-26
License: MIT + file LICENSE
Authors@R: c(
    person("Scott", "Chamberlain", role = "aut"),
    person("Ben", "Tupper", role = "ctb"),
    person("Salvador Jesús Fernández", "Bejarano", role = "ctb"),
    person("Roy", "Mendelssohn", role = c("cre", "ctb"),
    email = "roy.mendelssohn@noaa.gov")
    )
URL: https://docs.ropensci.org/rerddap/,
        https://github.com/ropensci/rerddap
BugReports: https://github.com/ropensci/rerddap/issues
LazyData: true
Depends: R (>= 4.00)
Encoding: UTF-8
Language: en-US
Imports: crul (>= 0.7.4), dplyr (>= 0.5.0), data.table (>= 1.12.0),
        digest, hoardr (>= 0.5.2), jsonlite (>= 1.6), lubridate,
        methods, nanoparquet, ncdf4 (>= 1.16), tibble, utils, xml2 (>=
        1.2.0)
Suggests: knitr, plotly, rmarkdown
VignetteBuilder: knitr
X-schema.org-applicationCategory: Climate
X-schema.org-keywords: earth, science, climate, precipitation,
        temperature, storm, buoy, NOAA
X-schema.org-isPartOf: https://ropensci.org
Config/testthat/edition: 3
Config/roxygen2/version: 8.0.0.9000
NeedsCompilation: no
Packaged: 2026-07-02 18:20:51 UTC; rmendels
Author: Scott Chamberlain [aut],
  Ben Tupper [ctb],
  Salvador Jesús Fernández Bejarano [ctb],
  Roy Mendelssohn [cre, ctb]
Maintainer: Roy Mendelssohn <roy.mendelssohn@noaa.gov>
Repository: CRAN
Date/Publication: 2026-07-03 03:00:02 UTC
