# Generated by roxygen2: do not edit by hand

S3method(as.info,character)
S3method(as.info,info)
S3method(browse,character)
S3method(browse,default)
S3method(browse,griddap_csv)
S3method(browse,griddap_nc)
S3method(browse,info)
S3method(browse,tabledap)
S3method(cache_delete,character)
S3method(cache_delete,griddap_csv)
S3method(cache_delete,griddap_nc)
S3method(cache_delete,list)
S3method(cache_delete,tabledap)
S3method(cache_details,character)
S3method(cache_details,default)
S3method(cache_details,griddap_csv)
S3method(cache_details,griddap_nc)
S3method(cache_details,list)
S3method(cache_details,tabledap)
S3method(print,ed_search)
S3method(print,ed_search_adv)
S3method(print,griddap_csv)
S3method(print,griddap_nc)
S3method(print,griddap_parquet)
S3method(print,info)
S3method(print,rerddap_cache)
S3method(print,rerddap_cache_info)
S3method(print,tabledap)
export(as.info)
export(browse)
export(cache_delete)
export(cache_delete_all)
export(cache_details)
export(cache_info)
export(cache_list)
export(cache_setup)
export(convert_time)
export(convert_units)
export(disk)
export(ed_datasets)
export(ed_search)
export(ed_search_adv)
export(eurl)
export(fipscounty)
export(global_search)
export(griddap)
export(info)
export(key_words)
export(memory)
export(servers)
export(tabledap)
export(version)
importFrom(crul,HttpClient)
importFrom(data.table,rbindlist)
importFrom(digest,digest)
importFrom(dplyr,as_data_frame)
importFrom(dplyr,bind_rows)
importFrom(hoardr,hoard)
importFrom(jsonlite,fromJSON)
importFrom(ncdf4,nc_close)
importFrom(ncdf4,nc_open)
importFrom(ncdf4,ncvar_get)
importFrom(utils,URLencode)
importFrom(utils,head)
importFrom(utils,read.csv)
importFrom(utils,read.delim)
importFrom(xml2,read_html)
importFrom(xml2,xml_find_all)
importFrom(xml2,xml_find_first)
importFrom(xml2,xml_text)
