#' cmocean colors
#' The cmocean color palette by Kristen Thyng as implemented in the R package "oce"
#'
#' str(colors)
#' List of 13
#'  $ viridis
#'  $ cdom
#'  $ chlorophyll
#'  $ density
#'  $ freesurface
#'  $ oxygen
#'  $ par
#'  $ phase
#'  $ salinity
#'  $ temperature
#'  $ turbidity
#'  $ velocity
#'  $ vorticity
"colors"
