% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rerddap-package.r
\docType{data}
\name{standardnames}
\alias{standardnames}
\title{standardnames}
\format{
A character vector
}
\description{
standardnames
}
\keyword{datasets}
