% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rerddap-package.r
\docType{data}
\name{ioos_categories}
\alias{ioos_categories}
\title{ioos_categories}
\format{
A character vector
}
\description{
ioos_categories
}
\keyword{datasets}
