% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/rerddap-package.r
\docType{data}
\name{longnames}
\alias{longnames}
\title{longnames}
\format{
A character vector
}
\description{
longnames
}
\keyword{datasets}
